module Rookout
  class ForkManager
    require "singleton"
    include Singleton

    def initialize
      @active = false
    end

    def activate!
      @active = true
    end

    def disable!
      @active = false
    end

    def active?
      @active
    end

    def fork_hook original_fork
      if block_given?
        original_fork.call do
          post_fork_child if active?
          yield
        end
      else
        res = original_fork.call
        post_fork_child if active? && !res
        res
      end
    end

    def post_fork_child
      require_relative "rookout_singleton"
      require_relative "interface"

      RookoutSingleton.instance.post_fork_clean
      Interface.instance.stop
      Interface.instance.start post_fork: true

      # Disable fork handler in child process
      disable!
    end
  end
end

alias _rookout_original_fork fork

def self.fork &block
  Rookout::ForkManager.instance.fork_hook method(:_rookout_original_fork), &block
end

def fork &block
  Rookout::ForkManager.instance.fork_hook method(:_rookout_original_fork), &block
end

module Kernel
  alias _rookout_original_fork fork

  def self.fork &block
    Rookout::ForkManager.instance.fork_hook method(:_rookout_original_fork), &block
  end
end

module Process
  alias _rookout_original_fork fork

  def self.fork &block
    Rookout::ForkManager.instance.fork_hook method(:_rookout_original_fork), &block
  end
end
