module Rookout
  module Augs
    module Locations
      require_relative "../../processor/rook_error"

      MAX_LOG_CACHE_SIZE = 10

      class Location
        def initialize output, aug
          @output = output
          @aug = aug
          @log_cache = []
        end

        def add_aug _trigger_services
          raise NotImplementedError
        end

        def id
          @aug.id
        end

        def execute frame, extracted
          UserWarnings.with self do
            begin
              @aug.execute frame, extracted, @output
            rescue SystemExit
              raise
            rescue Exception => e
              message = "Exception while processing Aug"
              error = Processor::RookError.new e, message
              notify_warning error
            end
          end
        end

        def notify_active
          send_rule_status :Active
        end

        def notify_pending
          send_rule_status :Pending
        end

        def notify_removed
          send_rule_status :Deleted
        end

        def notify_error error
          send_rule_status :Error, error
        end

        def notify_warning error
          return if silence_log? error

          Logger.instance.warning error.message

          # For easier testing
          return if @output.nil?
          @output.send_rule_status id, :Warning, error
        end

        private

        def silence_log? error
          return true if @log_cache.length >= MAX_LOG_CACHE_SIZE || @log_cache.include?(error.message)

          @log_cache << error.message
          false
        end

        def send_rule_status status, error = nil
          return if @status == status

          Logger.instance.info "Updating rule status for #{@id} to #{status}"
          @status = status

          # For easier testing
          return if @output.nil?
          @output.send_rule_status id, status, error
        end
      end
    end
  end
end
