module Rookout
  module ComWs
    module Git
      GIT_FOLDER = ".git".freeze
      GIT_HEAD = "HEAD".freeze
      GIT_CONFIG = "config".freeze

      REMOTE_ORIGINS = /\[remote "origin"\]\s*url\s*=\s*(\S*)/.freeze

      def find_root path
        # Return if found
        return path if git? path

        # Get next, fail if there's no next
        next_path = File.dirname path
        return nil if next_path == path

        # Recursive call
        find_root next_path
      end

      def revision path
        follow_sym_links File.join(path, GIT_FOLDER), GIT_HEAD
      end

      def remote_origin path
        git_config_path = File.join path, GIT_FOLDER, GIT_CONFIG
        git_config_contents = File.read git_config_path
        match = git_config_contents.match REMOTE_ORIGINS
        return "" if match.captures.empty?

        match.captures[0]
      end

      def git? path
        potential_git_folder = File.join path, GIT_FOLDER
        File.directory? potential_git_folder
      end

      def follow_sym_links root, link
        link_path = File.join root, link
        link_contents = File.read link_path

        if link_contents.start_with? "ref:"
          next_link = (link_contents.split " ")[1].strip
          follow_sym_links root, next_link
        else
          link_contents.strip
        end
      end
    end
  end
end
