module Rookout
  module ComWs
    class TokenBucket
      def initialize limit, interval_seconds, &do_once_when_exhausted
        @initial_limit = limit
        @remaining = limit
        @last_reset = Time.new
        @interval = interval_seconds
        @do_once_when_exhausted = do_once_when_exhausted
        @do_once_when_exhausted_performed = false
      end

      def exhausted?
        if Time.new - @last_reset > @interval
          @last_reset = Time.new
          @remaining = @initial_limit
          @do_once_when_exhausted_performed = false
        end

        @remaining < 0
      end

      def if_available
        @remaining -= 1
        if exhausted?
          return if @do_once_when_exhausted_performed || @do_once_when_exhausted.nil?

          @do_once_when_exhausted.call
          @do_once_when_exhausted_performed = true
        else
          yield
        end
      end
    end
  end
end
