module Rookout
  module Processor
    module Namespaces
      require "English"

      require_relative "../../exceptions"

      require_relative "namespace"
      require_relative "ruby_object_namespace"

      class RubyUtilsNamespace < Namespace
        def initialize
          super
        end

        def call_method name, args
          case name
          when "exception"
            RubyObjectNamespace.new $ERROR_INFO
          when "module"
            find_module args
          when "env"
            raise Exceptions::RookKeyNotFound, args unless ENV.include? args
            RubyObjectNamespace.new ENV[args]
          when "thread_id"
            RubyObjectNamespace.new Thread.current.__id__
          when "thread_name"
            RubyObjectNamespace.new Thread.current.name
          when "threads"
            RubyObjectNamespace.new Thread.list
          when "thread_tracebacks"
            thread_tracebacks
          else
            super
          end
        end

        def read_attribute _name
          RubyObjectNamespace.new nil
        end

        def read_key _key
          RubyObjectNamespace.new nil
        end

        private

        def find_module args
          RubyObjectNamespace.new Object.const_get(args)
        rescue NameError
          raise Exceptions::RookKeyNotFound, args
        end

        def thread_tracebacks
          result = Thread.list.map do |thread|
            { id: thread.__id__,
              name: thread.name,
              traceback: thread.backtrace }
          end
          RubyObjectNamespace.new result
        end
      end
    end
  end
end
