module Rookout
  module Processor
    module Namespaces
      require_relative "namespace"
      require_relative "frame_namespace"

      require_relative "../../protobuf/variant_pb"

      class TracebackNamespace < Namespace
        def initialize backtrace, offset, depth
          @backtrace = backtrace
          @offset = offset
          @depth = depth
        end

        def read_key key
          FrameNamespace.new @backtrace[key + @offset]
        end

        def dump _log_object_errors
          traceback = Com::Rookout::Variant::Traceback.new

          @depth.times do |i|
            position = i + @offset
            break if position >= @backtrace.length
            frame = @backtrace[position]

            code_object = Com::Rookout::Variant::CodeObject.new filename: frame.source_location[0],
                                                                lineno: frame.source_location[1],
                                                                name: frame.eval("__method__").to_s

            traceback.locations << code_object
          end

          Com::Rookout::Variant.new variant_type: :VARIANT_TRACEBACK, traceback: traceback
        end
      end
    end
  end
end
