module Rookout
  module Processor
    module Operations
      require_relative "operation"

      require_relative "../namespaces/ruby_object_namespace"

      require_relative "../rook_error"
      require_relative "../../logger"
      require_relative "../../user_warnings"

      class SetOperation < Operation
        def initialize configuration, factory
          @paths = []

          configuration["paths"].each do |key, value|
            begin
              dest_path = factory.create_path key
              source_path = factory.create_path value
              @paths.push [dest_path, source_path]
            rescue StandardError => e
              message = "Failed to load dest:source path pair"
              Logger.instance.exception message, e
              warning = RookError.new e, message
              UserWarnings.notify_warning warning
            end
          end
        end

        def execute namespace
          @paths.each do |dest_path, source_path|
            begin
              value = source_path.read_from namespace
              if value.is_a?(Namespaces::RubyObjectNamespace) &&
                 value.dump_config == Namespaces::OBJECT_DUMP_CONFIG_DEFAULT
                value.tailor_limits!
              end

              dest_path.write_to namespace, value
            rescue StandardError => e
              message = "Failed to execute dest:source path pair"
              Logger.instance.exception message, e
              warning = RookError.new e, message
              UserWarnings.notify_warning warning
            end
          end
        end
      end
    end
  end
end
