module Rookout
  module Processor
    module Paths
      require_relative "path"

      require_relative "../../exceptions"

      require_relative "../namespaces/ruby_object_namespace"

      require_relative "canopy/maps"
      require_relative "canopy/actions"
      require_relative "canopy/markers"

      class ArithmeticPath < Path
        def initialize path
          if path.is_a? Hash
            path = path["path"] || path[:path]
          end

          @raw_path = path

          if path.start_with?("NOT(") && path.end_with?(")")
            @path = path["NOT(".length..-2]
            @negation = true
          else
            @path = path
            @negation = false
          end

          raise RookInvalidArithmeticPath, @raw_path if @path.empty?
        end

        def read_from namespace
          begin
            result = Maps.parse @path, actions: Canopy::Actions.new(namespace)
          rescue Maps::ParseError => e
            raise RookInvalidArithmeticPath.new(@path, e)
          end

          normalize result
        end

        def write_to namespace, value
          context = Canopy::Actions.new namespace

          begin
            # initialize operations chain - by parsing the expression - ignore return value
            Maps.parse @path, actions: context
          rescue Maps::ParseError => e
            raise RookInvalidArithmeticPath.new(@path, e)
          end

          raise RookInvalidArithmeticPath, @raw_path if context.operations.empty?

          (context.operations.length - 1).times do |i|
            namespace = context.operations[i].read namespace, true
          end

          context.operations[context.operations.length - 1].write namespace, value
        end

        def normalize result
          if result.is_a? Canopy::List
            new_array = result.obj.map(&:obj)
            return Namespaces::RubyObjectNamespace.new new_array
          end

          if result.is_a? Canopy::NamespaceResult
            return result.obj
          end

          if result.is_a? Canopy::ToolExceptionMarker
            raise result.obj if result.obj.is_a? Exceptions::ToolException
            raise Exceptions::RookInvalidArithmeticPath.new(@raw_path, result.obj)
          end

          res = result.obj
          res = !res if [true, false].include?(res) && @negation
          Namespaces::RubyObjectNamespace.new res
        end
      end
    end
  end
end
