module Rookout
  module Processor
    module Paths
      module Canopy
        require "bigdecimal"

        LEVEL1 = ["*", "/"].freeze
        LEVEL2 = ["+", "-"].freeze
        LEVEL3 = ["<=", ">=", "!=", "=", "==", ">", "<", "LT", "GT", "LE", "GE", "EQ", "NE", "lt", "gt", "le", "ge",
                  "eq", "ne"].freeze
        LEVEL4 = ["in"].freeze
        LEVEL5 = ["or", "OR", "||", "and", "AND", "&&"].freeze

        ALL_LEVELS = [LEVEL1, LEVEL2, LEVEL3, LEVEL4, LEVEL5].freeze

        PRIMITIVES = [Array, Hash, TrueClass, FalseClass, Symbol, Integer, Float, Complex, Rational, BigDecimal,
                      String].freeze

        OPS_ALTERNATIVES = {
          "NE" => "!=",
          "=" => "==",
          "EQ" => "==",
          "LT" => "<",
          "GT" => ">",
          "GE" => ">=",
          "LE" => "<=",
          "AND" => "and",
          "OR" => "or",
          "&&" => "and",
          "IN" => "in",
          "||" => "or"
        }.freeze

        OPS_FUNCTIONS = {
          "+" => ->(a, b) { a + b },
          "-" => ->(a, b) { a - b },
          "/" => ->(a, b) { a / b },
          "*" => ->(a, b) { a * b },
          "<" => ->(a, b) { a < b },
          "<=" => ->(a, b) { a <= b },
          ">" => ->(a, b) { a > b },
          ">=" => ->(a, b) { a >= b },
          "!=" => ->(a, b) { a != b },
          "=" => ->(a, b) { a == b },
          "==" => ->(a, b) { a == b },
          "in" => lambda do |a, b|
            # If both are strings, just call include
            return b.include? a if b.is_a?(String) && a.is_a?(String)

            # If it's a dict, scan the keys, compensating for additional abstraction
            if b.is_a? Hash
              b.each_key do |key|
                if key.is_a? ObjectMarker
                  return true if key.obj == a
                elsif key == a
                  return true
                end
              end
              return false
            end

            # If it's an array, scan the value, compensating for additional abstraction
            if b.is_a? Array
              b.each do |value|
                if value.is_a? ObjectMarker
                  return true if value.obj == a
                elsif value == a
                  return true
                end
              end
              return false
            end

            false
          end,
          "and" => ->(a, b) { a && b },
          "or" => ->(a, b) { a || b }
        }.freeze
      end
    end
  end
end
