module Rookout
  module Processor
    require_relative "../protobuf/variant_pb"

    require_relative "namespace_serializer"
    require_relative "namespaces/ruby_utils_namespace"

    require_relative "../exceptions"

    class RookError
      def initialize exception, message = ""
        @exception = exception
        @traceback = exception.backtrace

        if @exception.is_a? Exceptions::ToolException
          raw_type = @exception.class.to_s
          @type = raw_type[raw_type.rindex(":") + 1..raw_type.length]
          @message = @exception.message
          @parameters = @exception.parameters
        else
          @type = "Unknown"
          @message = message
          @parameters = nil
        end
      end

      attr_reader :message, :exception, :parameters

      def dumps
        parameters = NamespaceSerializer.dump Namespaces::RubyObjectNamespace.new(@parameters), false
        exception = NamespaceSerializer.dump Namespaces::RubyObjectNamespace.new(@exception), false

        backtrace_string = ""
        unless @exception.backtrace.nil?
          backtrace_string = @exception.backtrace.join "\n\t"
        end
        backtrace_object = Namespaces::RubyObjectNamespace.new(backtrace_string).tailor_limits!
        traceback = NamespaceSerializer.dump backtrace_object, false

        Com::Rookout::Error.new message: @message,
                                type: @type,
                                parameters: parameters,
                                exc: exception,
                                traceback: traceback
      end
    end
  end
end
