module Rookout
  require_relative "logger"
  require_relative "trigger_services"
  require_relative "exceptions"

  require_relative "augs/augs_manager"

  require_relative "com_ws/agent_com_ws"
  require_relative "com_ws/output"
  require_relative "com_ws/command_handler"

  class RookoutSingleton
    require "singleton"
    include Singleton

    def initialize
      check_version_supported

      @output = ComWs::Output.new

      @services_started = false
      @services = TriggerServices.new

      @aug_manager = Augs::AugsManager.new @services, @output

      @agent_com = nil
    end

    def start_trigger_services
      return if @services_started

      @services.start
      @services_started = true
    end

    def stop_trigger_services
      return unless @services_started

      @services.close
      @services_started = false
    end

    def connect token: nil, host: nil, port: nil, proxy: nil, labels: [], async_start: false, fork: false
      raise Exceptions::RookInterfaceException, "Multiple connection attempts not supported!" unless @agent_com.nil?

      if fork
        require_relative "atfork"
        Rookout::ForkManager.instance.activate!
      end

      start_trigger_services

      Logger.instance.debug "Initiating AgentCom-\t#{host}:#{port}"

      @agent_com = ComWs::AgentComWs.new @output, host, port, proxy, token, labels
      @output.agent_com = @agent_com
      @command_handler = ComWs::CommandHandler.new @agent_com, @aug_manager

      @agent_com.connect
      return if async_start

      @agent_com.wait_for_ready
    end

    def flush
      @output.flush_messages if !@output.nil && !@agent_com.nil?
    end

    def post_fork_clean
      @agent_com.stop
      @agent_com = nil

      @command_handler = nil

      # We don't disable services because we will lose all loaded scripts
      @services.clear_augs
    end

    attr_reader :services

    private

    def check_version_supported
      raise Exceptions:: RookVersionNotSupported.new("platform", RUBY_ENGINE) unless RUBY_ENGINE == "ruby"
      raise Exceptions::RookVersionNotSupported.new("version", RUBY_VERSION) unless
          RUBY_VERSION.start_with?("2.7") || RUBY_VERSION.start_with?("2.6")
    end
  end
end
